/*
 * Name:    mp_camac.c
 *
 * Purpose: Python module interface to generic MX CAMAC functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_types.h"
#include "mx_camac.h"
#include "mx_handle.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_camac_command_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	char *mode;
	int slot, subaddress, function_code, Q, X;
	long long_data;
	mx_sint32_type data;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "liiils", &handle,
			&slot, &subaddress, &function_code, &long_data, &mode );

	if ( parse_status == 0 )
		return NULL;

	data = (mx_sint32_type) long_data;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	if ( strcmp( mode, "qwait" ) == 0 ) {

		/* Wait until Q == 1. */

		mx_camac_qwait( record,
				slot, subaddress, function_code,
				&data, &X );

		Q = 1;
	} else {
		mx_status = mx_camac( record,
				slot, subaddress, function_code,
				&data, &Q, &X );
	}

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the results back as a tuple of ( data, Q, X ) */

	result = Py_BuildValue( "lii", (long) data, Q, X );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_camac_get_lam_status_cmd(PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, lam_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_camac_get_lam_status( record, &lam_status );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", lam_status );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_camac_controller_command_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status, command;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li", &handle, &command );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_camac_controller_command( record, command );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

